@@ -7,6 +7,7 @@
 """
 
 from .data_model import KISAO_ALGORITHM_MAP, UpdatePolicy  # noqa: F401
+from biosimulators_utils.config import Config  # noqa: F401
 from biosimulators_utils.report.data_model import VariableResults
 from biosimulators_utils.sedml.data_model import (  # noqa: F401
     ModelLanguage, Simulation, UniformTimeCourseSimulation, Symbol)
@@ -19,15 +20,17 @@ from ginsim.gateway import japi as ginsim_japi
 import biolqm  # noqa: F401
 import biosimulators_utils.sedml.validation
 import biosimulators_utils.xml.utils
-import lxml.etree
+import lxml.etree  # noqa: F401
 import numpy
 import os
 import py4j.java_gateway  # noqa: F401
+import types  # noqa: F401
 
 __all__ = [
     'validate_simulation',
     'validate_time_course',
     'get_variable_target_xpath_ids',
+    'validate_variables',
     'read_model',
     'set_up_simulation',
     'exec_simulation',
@@ -96,22 +99,22 @@ def validate_time_course(simulation):
     return (errors, warnings)
 
 
-def get_variable_target_xpath_ids(variables, model_source):
+def get_variable_target_xpath_ids(variables, model_etree):
     """ Get the SBML-qual id for each XML XPath target of a SED-ML variable
 
     Args:
         variables (:obj:`list` of :obj:`Variable`): variables of data generators
-        model_source (:obj:`str`): path to model
+        model_etree (:obj:`lxml.etree._ElementTree`): element tree for model
 
     Returns:
         :obj:`dict`: dictionary that maps each variable target to the id of the
             corresponding qualitative species
     """
-    namespaces = biosimulators_utils.xml.utils.get_namespaces_for_xml_doc(lxml.etree.parse(model_source))
+    namespaces = biosimulators_utils.xml.utils.get_namespaces_for_xml_doc(model_etree)
 
-    return biosimulators_utils.sedml.validation.validate_variable_xpaths(
+    return biosimulators_utils.sedml.validation.validate_target_xpaths(
         variables,
-        model_source,
+        model_etree,
         attr={
             'namespace': {
                 'prefix': 'qual',
@@ -122,26 +125,76 @@ def get_variable_target_xpath_ids(variables, model_source):
     )
 
 
-def read_model(filename):
+def validate_variables(variables, model, model_language, target_xpath_ids, simulation):
+    """ Get the result of each SED-ML variable
+
+    Args:
+        variables (:obj:`list` of :obj:`Variable`): variables
+        model (:obj:`py4j.java_gateway.JavaObject`): bioLQM model
+        model_language (:obj:`str`): model language
+        target_xpath_ids (:obj:`dict`): dictionary that maps XPaths to the SBML qualitative ids
+            of the corresponding objects
+        simulation (:obj:`Simulation`): analysis
+    """
+    component_ids = set(component.getNodeID() for component in model.getComponents())
+    invalid_variables = []
+
+    for variable in variables:
+        if variable.symbol:
+            if not (isinstance(simulation, UniformTimeCourseSimulation) and variable.symbol == Symbol.time.value):
+                invalid_variables.append('{}: symbol: {}'.format(variable.id, variable.symbol))
+        else:
+            if model_language == ModelLanguage.SBML.value:
+                id = target_xpath_ids[variable.target]
+            else:
+                id = variable.target
+
+            if id not in component_ids:
+                invalid_variables.append('{}: target: {}'.format(variable.id, variable.target))
+
+    if invalid_variables:
+        valid_variables = []
+
+        if isinstance(simulation, UniformTimeCourseSimulation):
+            valid_variables.append('symbol: {}'.format(Symbol.time.value))
+
+        for component_id in component_ids:
+            if model_language == ModelLanguage.SBML.value:
+                valid_variables.append(
+                    "target: /sbml:sbml/sbml:model/qual:listOfQualitativeSpecies/qual:qualitativeSpecies[@id='{}']".format(component_id))
+            else:
+                valid_variables.append(
+                    'target: {}'.format(component_id))
+
+        raise ValueError((
+            'The following variables cannot be recorded:\n'
+            '  {}\n'
+            '\n'
+            'Variables with the following symbols and targets can be recorded:\n'
+            '  {}'
+        ).format(
+            '\n  '.join(sorted(invalid_variables)),
+            '\n  '.join(sorted(valid_variables)),
+        ))
+
+
+def read_model(filename, language):
     """ Read a model
 
     Args:
-        filename (:obj:`str`): path to model
+        language (:obj:`ModelLanguage`): language
 
     Returns:
         :obj:`py4j.java_gateway.JavaObject`: model
     """
+    if language == ModelLanguage.SBML:
+        format = 'sbml'
+    else:
+        format = None
+
     if not os.path.isfile(filename):
         raise FileNotFoundError('`{}` is not a file.'.format(filename))
 
-    format = None
-    try:
-        root = lxml.etree.parse(filename).getroot()
-        if root.tag.startswith('{http://www.sbml.org/sbml/'):
-            format = 'sbml'
-    except lxml.etree.XMLSyntaxError:
-        pass  # pragma: no cover
-
     model = ginsim_japi.lqm.load(filename, format)
 
     if model is None:
@@ -150,22 +203,23 @@ def read_model(filename):
     return model
 
 
-def set_up_simulation(simulation):
+def set_up_simulation(simulation, config=None):
     """ Set up an analysis
 
     Args:
         simulation (:obj:`Simulation`): analysis
+        config (:obj:`Config`, optional): configuration
 
     Returns:
         :obj:`tuple`:
 
             * :obj:`str`: KiSAO of algorithm to execute
             * :obj:`str`: name of the :obj:`biolqm` simulation/analysis method
-            * :obj:`list` of :obj:`str`: arguments for simulation method
+            * :obj:`types.LambdaType` of :obj:`Simulation` -> :obj:`list` of :obj:`str`: arguments for simulation method
     """
     # simulation algorithm
     alg_kisao_id = simulation.algorithm.kisao_id
-    alg_substitution_policy = get_algorithm_substitution_policy()
+    alg_substitution_policy = get_algorithm_substitution_policy(config=config)
     exec_kisao_id = get_preferred_substitute_algorithm_by_ids(
         alg_kisao_id, KISAO_ALGORITHM_MAP.keys(),
         substitution_policy=alg_substitution_policy)
@@ -175,7 +229,7 @@ def set_up_simulation(simulation):
             simulation.__class__.__name__, exec_kisao_id, alg_props['name']))
 
     method = alg_props['method']
-    method_args = alg_props['method_args'](simulation)
+    method_args = []
 
     # Apply the algorithm parameter changes specified by `simulation.algorithm.parameter_changes`
     if exec_kisao_id == alg_kisao_id:
@@ -212,7 +266,11 @@ def set_up_simulation(simulation):
             warn('Unsuported algorithm parameter `{}` was ignored.'.format(change.kisao_id), BioSimulatorsWarning)
 
     # return
-    return (exec_kisao_id, method, method_args)
+    return (
+        exec_kisao_id,
+        method,
+        lambda simulation: alg_props['method_args'](simulation) + method_args,
+    )
 
 
 def exec_simulation(method_name, model, args=None):
@@ -254,14 +312,10 @@ def get_variable_results(variables, model_language, target_xpath_ids, simulation
     for variable in variables:
         variable_results[variable.id] = numpy.full((n_states,), numpy.nan)
 
-    invalid_variables = []
     for i_state, state in enumerate(raw_results):
         for variable in variables:
             if variable.symbol:
-                if isinstance(simulation, UniformTimeCourseSimulation) and variable.symbol == Symbol.time.value:
-                    variable_results[variable.id][i_state] = i_state
-                else:
-                    invalid_variables.append('{}: symbol: {}'.format(variable.id, variable.symbol))
+                variable_results[variable.id][i_state] = i_state
 
             else:
                 if model_language == ModelLanguage.SBML.value:
@@ -269,13 +323,7 @@ def get_variable_results(variables, model_language, target_xpath_ids, simulation
                 else:
                     id = variable.target
 
-                variable_results[variable.id][i_state] = state.get(id, numpy.nan)
-                if i_state == 0 and numpy.isnan(variable_results[variable.id][i_state]):
-                    invalid_variables.append('{}: target: {}'.format(variable.id, variable.target))
-
-    if invalid_variables:
-        raise ValueError('The following variables could not recorded:\n  {}'.format(
-            '\n  '.join(sorted(invalid_variables))))
+                variable_results[variable.id][i_state] = state[id]
 
     if isinstance(simulation, UniformTimeCourseSimulation):
         for key in variable_results.keys():
